@@ -38,6 +38,7 @@
  * 07 Oct 11, jf@dockes.org, fix to add_to_collection()
  * 09 Jun 13, peterli@salk.edu, added code to cope with bold/italic
  			requests that are not properly nested
+ * 27 Jun 13, peterli@salk.edu, improved efficiency of the unnested code
  *--------------------------------------------------------------------*/
 
 #ifdef HAVE_CONFIG_H
@@ -884,6 +885,14 @@ void attr_pop_gen(AttrStack *stack, int *attr, char **param) {
   stack->tos--;
 }
 
+// Iterate through stack looking for findattr 
+int attr_find(AttrStack *stack, int findattr) {
+  int i;
+  for (i = 0; i <= stack->tos; ++i)
+    if (stack->attr_stack[i] == findattr) return i;
+  return -1;
+}
+
 // Pop attrs until we find the match, then push the non-matches back
 static AttrStack temp_stack;
 int attr_find_pop(int findattr) {
@@ -895,11 +904,13 @@ int attr_find_pop(int findattr) {
     return FALSE;
   }
 
+  // Check whether findattr is on stack at all; if not, just give up
+  if (attr_find(stack, findattr) < 0) return FALSE;
+
+  // Get findattr from stack, expressing endings.
+  // Store any non-matches to temp_stack.
   int attr = ATTR_NONE;
   char *param = NULL;
-
-  // Look for findattr on stack, expressing endings.
-  // Store any non-matches to temp_stack
   while (stack->tos >= 0) {
     attr_pop_gen(stack, &attr, &param);
     attr_express_end(attr, param);
@@ -907,7 +918,7 @@ int attr_find_pop(int findattr) {
     attr_push_gen(&temp_stack, attr, param);
   }
 
-  // Put the non-matches back, expressing beginnings
+  // Put the non-matches back, reexpressing beginnings
   while (temp_stack.tos >= 0) {
     attr_pop_gen(&temp_stack, &attr, &param);
     attr_push(attr, param);
@@ -916,5 +927,3 @@ int attr_find_pop(int findattr) {
   return TRUE;
 }
 #endif /* SUPPORT_UNNESTED */
-
-
