@@ -1,3 +1,6 @@
+#ifndef _ATTR_H_INCLUDED_
+#define _ATTR_H_INCLUDED_
+
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
    Copyright (C) 2000,2001,2004 by Zachary Smith
@@ -32,6 +35,8 @@
  * 18 Sep 01, tuorfa@yahoo.com: updates for AttrStack paradigm
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
+ * 09 Nov 08, arkadiusz.firus@gmail.com: adopt safe_printf & collection funcs
+ * 07 Oct 11, jf@dockes.org: removed unused protocols, added _ATTR_H_INCLUDED_
  *--------------------------------------------------------------------*/
 
 enum {
@@ -60,14 +65,20 @@ enum {
 	ATTR_DBL_STRIKE, 
 
 	ATTR_EXPAND,
+        ATTR_ENCODING,
 	/* ATTR_CONDENSE */
 };
 
+typedef struct _c
+{
+	int nr;
+	char *text;
+	struct _c *next;
+} Collection;
 
-
-extern void attr_push_core (int attr, char* param);
-
-extern void attr_pop_core (int attr);
+Collection *add_to_collection(Collection *col, int nr, char *text);
+char *get_from_collection(Collection *c, int nr);
+void free_collection(Collection *c);
 
 extern void attr_push(int attr, char* param);
 
@@ -85,12 +96,11 @@ extern void attr_pop_all();
 
 extern void attr_pop_dump();
 
-#if 1 /* daved 0.20.2 */
-
 char * attr_get_param(int attr);
 
-#endif
-
-
+int safe_printf(int nr, char *string, ...);
+char *assemble_string(char *string, int nr);
+#define TOO_MANY_ARGS "Tag name \"%s\" do not take so many arguments"
 
 
+#endif /* _ATTR_H_INCLUDED_ */
