@@ -1,3 +1,6 @@
+#ifndef _DEFS_H_INCLUDED_
+#define _DEFS_H_INCLUDED_
+
 
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
@@ -39,6 +42,7 @@
  * 17 Dec 07, daved@physiol.usyd.edu.au: added --noremap to usage - from
  *		David Santinoli
  * 09 Nov 08, arkadiusz.firus@gmail.com: updated usage
+ * 07 Oct 11, jf@dockes.org, added _DEFS_H_INCLUDED_
  *--------------------------------------------------------------------*/
 
 
@@ -76,3 +80,4 @@
 #define FONTDECOR_STR	"ZapfChancery"
 #define FONTTECH_STR	"Symbol"
 
+#endif /* _DEFS_H_INCLUDED_ */
