@@ -1,3 +1,6 @@
+#ifndef _HASH_H_INCLUDED_
+#define _HASH_H_INCLUDED_
+
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
    Copyright (C) 2000,2001,2004 by Zachary Smith
@@ -29,6 +32,8 @@
  * Changes:
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 06 Jan 06, marcossamaral@terra.com.br: changes hash_stats() 
+ * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
+ * 07 Oct 11, jf@dockes.org, added _HASH_H_INCLUDED_
  *--------------------------------------------------------------------*/
 
 
@@ -39,3 +44,5 @@ extern unsigned long hash_get_index (char *);
 extern char* hash_get_string (unsigned long );
 
 
+
+#endif /* _HASH_H_INCLUDED_ */
