@@ -37,8 +37,8 @@
  * 17 Dec 07, daved@physiol.usyd.edu.au: added support for --noremap from
  *		David Santinoli
  * 09 Nov 08, arkadiusz.firus@gmail.com: use iconv
- * 21 Aug 10, daved@physiol.usyd.edu.au: add code to deal with character
- 	doublets (see convert.c)
+ * 07 Oct 11, jf@dockes.org: replaced op_translate_doublet with
+ *	op_translate_buffer and other major changes
  *--------------------------------------------------------------------*/
 
 
@@ -58,18 +58,16 @@
 #include <string.h>
 #endif
 
+#include <arpa/inet.h>
+
 #include "malloc.h"
 #include "defs.h"
 #include "error.h"
 #include "output.h"
 #include "main.h"
 #include "convert.h"
-
-
-#ifndef HAVE_ATTR_H
 #include "attr.h"
-#define HAVE_ATTR_H
-#endif
+#include "util.h"
 
 /*========================================================================
  * Name:	op_create
@@ -108,364 +106,146 @@ op_free (OutputPersonality *op)
 }
 
 
-
-
 /*========================================================================
  * Name:	op_translate_char
- * Purpose:	Performs a translation of a character in the context of
- *		a given output personality.
- * Args:	OutputPersonality, character set#, character.
+ * Purpose:	Native encoding value->Unicode number as int
+ *    Then transform as specified by op (ie, entity, &#xxx;...)
+ *    Currently this is only called with values < 255 (single byte chars or
+ *    computation of Symbol code point in fields).
+ *    We cache a local iconv descriptor for efficiency.
+ * Args:	OutputPersonality, encoding, character.
  * Returns:	String.
  *=======================================================================*/
-
 char *
-op_translate_char (OutputPersonality *op, my_iconv_t cd, int ch)
+op_translate_char (OutputPersonality *op, char *encoding, int ch)
 {
-	short start;
-	char *result=NULL;
-	static char output_buffer[2]={ 0, 0 };
-	char *inbuf, *outbuf;
-	size_t inbytes = (ch / 256) + 1, outbytes = inbytes * 4, i;
+    static my_iconv_t m_desc = MY_ICONV_T_CLEAR;
+    static char *m_encoding = NULL;
+    char *result = NULL;
+    static char output_buffer[2]={ 0, 0 };
 
-	CHECK_PARAM_NOT_NULL(op);
+    CHECK_PARAM_NOT_NULL(op);
 
-	if (no_remap_mode == TRUE && ch < 256)
-	{
-		output_buffer[0]=ch;
-		result=output_buffer;
-	}
-	else
-	if (result == NULL)
-	{
-		inbuf = my_malloc(inbytes + 1);
-		outbuf = my_malloc(outbytes + 1);
+    if (ch < 0 || ch > 255) {
+        fprintf(stderr, "op_translate_char called with bad char 0x%x\n",
+                ch);
+        return NULL;
+    }
+    if (no_remap_mode == TRUE && ch < 256) {
+        /* Kept for historical reason, makes no sense in a multi-encoding
+           document */
+        output_buffer[0] = ch;
+        result = output_buffer;
+    } else {  
+        if (!m_encoding || strcasecmp(encoding, m_encoding)) {
+            // Using UTF-32BE to avoid BOM
+            if (my_iconv_is_valid(m_desc))
+                my_iconv_close(m_desc);
+            m_desc = my_iconv_open("UTF-32BE", encoding);
+            if (!my_iconv_is_valid(m_desc)) {
+                return NULL;
+            }
+            if (m_encoding)
+                my_free(m_encoding);
+            m_encoding = my_strdup(encoding);
+        }
+        size_t inbytes = 1, outbytes = 4;
+        char inbuf[1];
+        uint32_t outbuf;
+        char *icp = inbuf, *ocp = (char*)&outbuf;
+        int unicode;
+        inbuf[0] = ch;
 
-		for (i = inbytes - 1; ch > 255; i--)
-		{
-			inbuf[i] = ch % 256;
-			ch /= 256;
-		}
+        if (my_iconv(m_desc, &icp, &inbytes, &ocp, &outbytes) == -1)
+        {
+/*            fprintf(stderr, "unrtf: iconv error %s->UTF-32BE for 0x%x\n",
+	      encoding, ch);*/
+            return NULL;
+        }
 
-		inbuf[0] = ch;
-		inbuf[inbytes] = '\0';
-		i = outbytes;
-		if (!my_iconv_is_valid(cd))
-		{
-			cd = my_iconv_open("UTF-8", "cp1252");
-
-			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
-			{
-#if 1 /* daved */
-				fprintf(stderr, "unrtf: Error in executing iconv1\n");
-#else
-				fprintf(stderr, "unrtf: Error in executing iconv1\n");
+        /* Conversion from UTF-32BE string to int */
+        unicode = ntohl(outbuf);
+#if 0
+        if (unicode != ch)
+            fprintf(stderr, "op_translate: 0x%x -> unicode 0x%x\n", 
+                    (unsigned)ch, (unsigned)unicode);
 #endif
-				return NULL;
-			}
-			
-			my_iconv_close(cd);			
-		}
-		else
+        /* Output Unicode value as decided by output personality */
+        result = get_alias(op, unicode); /* Try for ie, entity */
+        if (result == NULL) {
+            if (ch > 127 && op->unisymbol_print) {
+                /* TODO: memory leak here. Impossible to fix easily as we can 
+                   return either a static or allocated buffer */
+                result = assemble_string(op->unisymbol_print, unicode);
+            } else {
+                /* Single byte unicode char needs no translation ever?*/
+                output_buffer[0] = ch;
+                result = output_buffer;
+            }
+        }
+    }
 
-			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
-			{
-				fprintf(stderr, "unrtf: Error in executing iconv\n");
-				return NULL;
-			}
-		*outbuf = '\0';
-		outbuf -= i - outbytes;
-/* Conversion from string to utf8 code number */
-		inbytes = 0;
-
-		for (i = 0; outbuf[i] != '\0'; i++)
-			inbytes++;
-
-		ch = 0;
-
-		for (i = 0; i < inbytes; i++)
-		{
-			if (i == 0)
-				switch (inbytes)
-				{
-					case 1:
-						ch =  outbuf[0];
-						break;
-					case 2:
-						ch = (unsigned char) outbuf[0] - 192;
-						break;
-					case 3:
-						ch = (unsigned char) outbuf[0] - 224;
-						break;
-					case 4:
-						ch = (unsigned char) outbuf[0] - 240;
-						break;
-				}
-			else
-				ch = (ch * 64) + ((unsigned char) outbuf[i] - 128);
-		}
-/* End of conversion*/
-
-		result = get_alias(op, ch);
-
-		if (result == NULL)
-			if (ch > 127 && op->unisymbol_print)
-				result = assemble_string(op->unisymbol_print, ch);
-			else
-				result = outbuf;
-	}
-
-	return result;
+    return result;
 }
 
-#if 0 /* daved - 0.21.2 */
 /*========================================================================
- * Name:	op_translate_str
- * Purpose:	Performs a translation of a doublet the context of
- *		a given output personality.
- * Args:	OutputPersonality, character set#, character.
- * Returns:	String.
+ * Name:	op_translate_buffer
+ * Purpose:	Translate buffer built out of \' commands in convert.c
+ *   The data comes in as UTF-32BE, which we change to unicode positions 
+ *   (just the byte order), then transform according to outputpersonality
+ *   TOBEDONE: have a way to specify direct UTF-8 output for a personality
+ *    (then produce utf-8 in convert.c and do nothing special here.
+ *  
+ * Args:	OutputPersonality, encoding, character.
+ * Returns:	malloced string.
  *=======================================================================*/
-
 char *
-op_translate_str (OutputPersonality *op, my_iconv_t cd, int * doublet)
+op_translate_buffer(OutputPersonality *op, char *buffer, int bytescnt)
 {
-	short start;
-	char *result=NULL;
-	static char output_buffer[3]={ 0, 0, 0 };
-	unsigned char *inbuf;
-	char *outbuf;
-	int	ch;
+    int wordcnt = bytescnt / 4;
+    int i;
+    int opos = 0;
+    char *out = 0;
+    int outsz;
+    if ((out = (char *)allocmem(out, 1, &outsz, 1000, 100000)) == NULL) {
+        fprintf(stderr, "unrtf: out of memory\n");
+        return NULL;
+    }
 
-	fprintf(stderr, "doublet = %o %o\n", *doublet, *(doublet+1));
-#if 0
-	size_t inbytes = (ch / 256) + 1, outbytes = inbytes * 4, i;
-#else
-	size_t inbytes = 3, outbytes = inbytes * 4, i;
-#endif
+    for (i = 0; i < wordcnt; i++) {
+        char tbuf[2];
+        char *result;
+        int incsz;
+        int unicode = ntohl(*(int*)(buffer + 4 * i));
 
-	CHECK_PARAM_NOT_NULL(op);
-
-#if 0
-	if (no_remap_mode == TRUE && ch < 256)
-	{
-		output_buffer[0]=ch;
-		result=output_buffer;
-	}
-	else
-	if (result == NULL)
-	{
-#endif
-		inbuf = (char *) my_malloc(inbytes + 1);
-		*inbuf = *doublet;
-		*(inbuf+1) = (unsigned char *) *(doublet+1);
-		*(inbuf+2) = 0;
-		fprintf(stderr, "inbuf = %o %o\n", *inbuf, *(inbuf+1));
-		outbuf = my_malloc(outbytes + 1);
-
-#if 0
-
-		for (i = inbytes - 1; ch > 255; i--)
-		{
-			inbuf[i] = ch % 256;
-			ch /= 256;
-		}
-
-		inbuf[0] = ch;
-		inbuf[inbytes] = '\0';
-#endif
-		i = outbytes;
-		if (!my_iconv_is_valid(cd))
-		{
-			cd = my_iconv_open("UTF-8", "cp1252");
-
-			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
-			{
-				fprintf(stderr, "unrtf: Error in executing iconv1\n");
-				return NULL;
-			}
-			
-			my_iconv_close(cd);			
-		}
-		else
-		{
-
-			if (my_iconv(cd, &inbuf, &inbytes, &outbuf, &outbytes) == -1)
-			{
-#if 1 /* daved */
-				fprintf(stderr, "unrtf: Error in executing iconv - inbytes=%d inbuf[0]=%o inbuf[1]=%o\n", inbytes, inbuf[0], inbuf[1]);
-#else
-				fprintf(stderr, "unrtf: Error in executing iconv\n");
-#endif
-				return NULL;
-			}
-		}
-		*outbuf = '\0';
-		outbuf -= i - outbytes;
-/* Conversion from string to utf8 code number */
-		inbytes = 0;
-
-		for (i = 0; outbuf[i] != '\0'; i++)
-			inbytes++;
-
-		ch = 0;
-
-		for (i = 0; i < inbytes; i++)
-		{
-			if (i == 0)
-				switch (inbytes)
-				{
-					case 1:
-						ch =  outbuf[0];
-						break;
-					case 2:
-						ch = (unsigned char) outbuf[0] - 192;
-						break;
-					case 3:
-						ch = (unsigned char) outbuf[0] - 224;
-						break;
-					case 4:
-						ch = (unsigned char) outbuf[0] - 240;
-						break;
-				}
-			else
-				ch = (ch * 64) + ((unsigned char) outbuf[i] - 128);
-		}
-/* End of conversion*/
-
-		result = get_alias(op, ch);
-
-		if (result == NULL)
-			if (ch > 127 && op->unisymbol_print)
-				result = assemble_string(op->unisymbol_print, ch);
-			else
-				result = outbuf;
-#if 0 /* daved 0.21.2 */
-	}
-#endif
-
-	return result;
+        result = get_alias(op, unicode); /* Try for ie, entity */
+        if (result != NULL) {
+            incsz = strlen(result);
+        } else {
+            if (unicode > 127 && op->unisymbol_print) {
+                /* TODO: fix memory leak */
+                result = assemble_string(op->unisymbol_print, unicode);
+                incsz = strlen(result);
+            } else {
+                /* Single byte unicode char needs no translation ever?*/
+                tbuf[0] = unicode;
+                tbuf[1] = 0;
+                result = tbuf;
+                incsz = 1;
+            }
+        }
+        if (opos + incsz + 1 >= outsz) {
+            if ((out = (char *)allocmem(out, 1, &outsz, 1000, 100000))) {
+                fprintf(stderr, "unrtf: out of memory\n");
+                return NULL;
+            }
+        }
+        strcpy(out + opos, result);
+        opos += incsz;
+    }
+    return out;
 }
-#endif
 
-#if 1 /* daved - 0.21.2 */
-/*========================================================================
- * Name:	op_translate_doublet
- * Purpose:	Performs a translation of a doublet the context of
- *		a given output personality.
- * Args:	OutputPersonality, character set#, character byte 1 and 2
- * Returns:	String.
- *=======================================================================*/
-
-char *
-op_translate_doublet (OutputPersonality *op, my_iconv_t cd, int ch1, int ch2)
-{
-	short start;
-	char *result=NULL;
-	static char output_buffer[3]={ 0, 0, 0 };
-	unsigned char *inbuf;
-	char *outbuf;
-	int	ch;
-
-#if 0
-	size_t inbytes = (ch / 256) + 1, outbytes = inbytes * 4, i;
-#else
-	size_t inbytes = 3, outbytes = inbytes * 4, i;
-#if 0
-	fprintf(stderr, "doublet = %o %o\n", ch1, ch2);
-#endif
-#endif
-
-	CHECK_PARAM_NOT_NULL(op);
-
-	if (no_remap_mode == TRUE && ch < 256)
-	{
-		output_buffer[0]=ch1;
-		output_buffer[1]=ch2;
-		result=output_buffer;
-	}
-	else
-	if (result == NULL)
-	{
-		inbuf = (char *) my_malloc(inbytes + 1);
-		*inbuf = ch1;
-		*(inbuf+1) = ch2;
-		*(inbuf+2) = 0;
-#if 0
-		fprintf(stderr, "inbuf = %o %o\n", *inbuf, *(inbuf+1));
-#endif
-		outbuf = my_malloc(outbytes + 1);
-
-		i = outbytes;
-		if (!my_iconv_is_valid(cd))
-		{
-			cd = my_iconv_open("UTF-8", "cp1252");
-
-			if (my_iconv(cd, (char **)&inbuf, (size_t *)&inbytes, (char **)&outbuf, &outbytes) == -1)
-			{
-				fprintf(stderr, "unrtf: Error in executing iconv1\n");
-				return NULL;
-			}
-			
-			my_iconv_close(cd);			
-		}
-		else
-		{
-
-			if (my_iconv(cd, (char **)&inbuf, (size_t *)&inbytes, (char **)&outbuf, &outbytes) == -1)
-			{
-#if 1 /* daved */
-				fprintf(stderr, "unrtf: Error in executing iconv - inbytes=%d inbuf[0]=%o inbuf[1]=%o\n", inbytes, inbuf[0], inbuf[1]);
-#else
-				fprintf(stderr, "unrtf: Error in executing iconv\n");
-#endif
-				return NULL;
-			}
-		}
-		*outbuf = '\0';
-		outbuf -= i - outbytes;
-/* Conversion from string to utf8 code number */
-		inbytes = 0;
-
-		for (i = 0; outbuf[i] != '\0'; i++)
-			inbytes++;
-
-		ch = 0;
-
-		for (i = 0; i < inbytes; i++)
-		{
-			if (i == 0)
-				switch (inbytes)
-				{
-					case 1:
-						ch =  outbuf[0];
-						break;
-					case 2:
-						ch = (unsigned char) outbuf[0] - 192;
-						break;
-					case 3:
-						ch = (unsigned char) outbuf[0] - 224;
-						break;
-					case 4:
-						ch = (unsigned char) outbuf[0] - 240;
-						break;
-				}
-			else
-				ch = (ch * 64) + ((unsigned char) outbuf[i] - 128);
-		}
-/* End of conversion*/
-
-		result = get_alias(op, ch);
-
-		if (result == NULL)
-			if (ch > 127 && op->unisymbol_print)
-				result = assemble_string(op->unisymbol_print, ch);
-			else
-				result = outbuf;
-	}
-
-	return result;
-}
-#endif
 /*========================================================================
  * Name:	op_begin_std_fontsize 
  * Purpose:	Prints whatever is necessary to perform a change in the
