@@ -1,3 +1,6 @@
+#ifndef _OUTPUT_H_INCLUDED_
+#define _OUTPUT_H_INCLUDED_
+
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
    Copyright (C) 2000,2001,2004 by Zachary Smith
@@ -30,25 +33,15 @@
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
  * 09 Nov 08, arkadiusz.firus@gmail.com: use iconv
+ * 07 Oct 11, jf@dockes.org: added _OUTPUT_H_INCLUDED_, argument changes
+ *	for op_translate_char etc. consistent with convert.c and output.c
  *--------------------------------------------------------------------*/
 
-#ifndef _OUTPUT
-
-#ifndef HAVE_ATTR_H
 #include "attr.h"
-#define HAVE_ATTR_H
-#endif
-
 #include "my_iconv.h"
 
 typedef Collection Aliases;
 
-typedef struct {
-	int cp;
-	unsigned short chars[128];
-} CodepageInfo;
-
-
 typedef struct {
 	char *comment_begin;
 	char *comment_end;
@@ -253,11 +246,9 @@ OutputPersonality;
 
 extern OutputPersonality* op_create(void);
 extern void op_free (OutputPersonality*);
-extern char* op_translate_char (OutputPersonality *op, my_iconv_t cd, int ch);
-#if 1 /* daved - 0.21.2 */
-extern char* op_translate_str (OutputPersonality *op, my_iconv_t cd, int * doublet);
-extern char* op_translate_doublet (OutputPersonality *op, my_iconv_t cd, int ch1, int ch2);
-#endif
+extern char *op_translate_char (OutputPersonality *op, char *encoding, int ch);
+extern char *op_translate_buffer(OutputPersonality *op, char *buffer, 
+                                 int bytescnt);
 
 extern void op_begin_std_fontsize (OutputPersonality*, int);
 extern void op_end_std_fontsize (OutputPersonality*, int);
@@ -265,6 +256,5 @@ extern void op_end_std_fontsize (OutputPersonality*, int);
 void add_alias(OutputPersonality *op, int nr, char *text);
 char *get_alias(OutputPersonality *op, int nr);
 
-#define _OUTPUT
-#endif
 
+#endif /* _OUTPUT_H_INCLUDED_ */
