@@ -1,3 +1,5 @@
+#ifndef _UTIL_H_INCLUDED_
+#define _UTIL_H_INCLUDED_
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
    Copyright (C) 2000,2001,2004 by Zachary Smith
@@ -30,14 +32,27 @@
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
  * 09 Nov 08, arkadiusz.firus@gmail.com: added leave_line
+ * 25 Sep 11, jf@dockes.org: add ifndef _UTIL_H_INCLUDED_ and allocmem
  *--------------------------------------------------------------------*/
 
-#ifndef HAVE_STDIO_H
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
-#define HAVE_STDIO_H
 #endif
 
 extern int h2toi (char *);
 
 void leave_line (FILE *f);
 extern char *concatenate (const char *, const char *);
+
+/* Help managing a growing contiguous array */
+extern void *
+allocmem
+(
+	void 	*cp,	/* The array to grow. Can be NULL */
+	int	 sz,	/* Unit size in bytes */
+	int 	*np,    /* Pointer to current allocation number (units) */
+	int	 min,   /* Number of units to allocate the first time */
+	int	 maxinc /* Maximum increment in units (maxinc*sz bytes) */
+);
+
+#endif /* _UTIL_H_INCLUDED_ */
