@@ -1,3 +1,6 @@
+#ifndef _WORD_H_INCLUDED_
+#define _WORD_H_INCLUDED_
+
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
    Copyright (C) 2000,2001,2004 by Zachary Smith
@@ -30,11 +33,9 @@
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
  * 09 Nov 08, arkadiusz.firus@gmail.com: add word_optimise
+ * 25 Sep 11, jf@dockes.org: reposition top #ifndef
  *--------------------------------------------------------------------*/
 
-#ifndef _WORD
-#define _WORD
-
 typedef struct _w {
 	unsigned long hash_index;
 	struct _w * next;
@@ -62,5 +63,4 @@ typedef struct _t {
 	{"", 0}\
 }
 
-#define _WORD
-#endif
+#endif /* _WORD_H_INCLUDED_ */
